package clinical.tools.dbadmin;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jdom.Element;

import clinical.utils.Assertion;
import clinical.utils.FileUtils;

/**
 * 
 * @author I. Burak Ozyurt
 * @version $Id: Utils.java 167 2010-02-09 00:57:41Z bozyurt $
 */
public class Utils {
	public static boolean isNumber(String s) {
		try {
			Double.parseDouble(s.trim());
		} catch (NumberFormatException e) {
			return false;
		}
		return true;
	}

	public static int toInt(String s) {
		return Integer.parseInt(s.trim());
	}

	public static List<String> filterPaths(List<String> paths,
			String[] protocolIds) {
		Pattern[] patterns = prepRegexPatterns(protocolIds);
		Set<String> filteredPaths = new HashSet<String>();
		for (String path : paths) {
			if (anyMatch(path, patterns)) {
				String strippedPath = stripAfterMatch(path, patterns);
				filteredPaths.add(strippedPath);
			}
		}
		return new ArrayList<String>(filteredPaths);
	}

	public static Pattern[] prepRegexPatterns(String[] protocolIds) {
		Pattern[] patterns = new Pattern[protocolIds.length];
		for (int i = 0; i < protocolIds.length; i++) {
			patterns[i] = Pattern.compile("\\b" + protocolIds[i] + "\\b");
		}
		return patterns;
	}

	public static String stripAfterMatch(String path, Pattern[] patterns) {
		for (Pattern p : patterns) {
			Matcher m = p.matcher(path);
			if (m.find()) {
				return path.substring(0, m.end());
			}
		}
		return path;
	}

	public static boolean anyMatch(String text, Pattern[] patterns) {
		for (Pattern p : patterns) {
			if (p.matcher(text).find()) {
				return true;
			}
		}
		return false;
	}

	public static String getDir(String path) {
		int idx = path.lastIndexOf('/');
		if (idx != -1) {
			return path.substring(0, idx);
		}
		return path;
	}

	public static Map<String, Object> prepArgMap(Map<String, Object> amap,
			boolean removeDateFields) {
		Map<String, Object> argMap = new HashMap<String, Object>(17);
		for (String key : amap.keySet()) {
			String value = amap.get(key).toString();
			if (removeDateFields && key.indexOf("modtime") != -1) {
				continue;
			} else if (value != null) {
				if (value.startsWith("00") || !Utils.isNumber(value)) {
					argMap.put(key, "'" + value + "'");
				} else {
					argMap.put(key, value);
				}
			}
		}
		return argMap;
	}
	
	public static Map<String, String> getDB(String usersXmlFile) throws Exception {
		Map<String,String> props = new HashMap<String, String>(17);
		Element rootEl = FileUtils.loadXML(usersXmlFile);
		List<?> dbEls = rootEl.getChild("databases").getChildren("database");
		Element defDBEl = null;
		for(Object o : dbEls) {
			Element dbEl = (Element) o;
			if (dbEl.getAttributeValue("default").equalsIgnoreCase("true")) {
				defDBEl = dbEl;
				break;
			}
		}
		Assertion.assertNotNull(defDBEl);
		String dbID = defDBEl.getAttributeValue("id");
		List<?> duEls = rootEl.getChild("dbusers").getChildren("dbuser");
		for(Object o : duEls) {
			Element duEl = (Element) o;
			String dbid = duEl.getAttributeValue("dbid");
			if (dbid.equals(dbID)) {
				props.put("db.type", duEl.getChildTextTrim("db-type"));
				props.put("db.url", duEl.getChildTextTrim("db-url"));
				props.put("db.user", duEl.getAttributeValue("name"));
				props.put("db.pwd", duEl.getAttributeValue("pwd"));
			}
		}
		
		
		return props;
		
	}
}
