DROP TABLE nc_assessmentData CASCADE CONSTRAINTS
;
-- 
-- TABLE: nc_assessmentData 
--

CREATE TABLE nc_assessmentData(
    tableID                         NUMBER(38, 0)    NOT NULL,
    uniqueID                        NUMBER(38, 0)    NOT NULL,
    scoreOrder                      NUMBER(38, 0)    NOT NULL,
    owner                           NUMBER(38, 0)    NOT NULL,
    modTime                         DATE             NOT NULL,
    modUser                         NUMBER(38, 0)    NOT NULL,
    assessmentID                    NUMBER(38, 0)    NOT NULL,
    scoreName                       VARCHAR2(128)    NOT NULL,
    scoreType                       VARCHAR2(64)     NOT NULL,
    textValue                       VARCHAR2(255),
    textNormValue                   VARCHAR2(255),
    encrypted                       NUMBER(1, 0)     NOT NULL,
    comments                        CLOB,
    nc_storedAssessment_uniqueID    NUMBER(38, 0)    NOT NULL,
    isValidated                     NUMBER(1, 0)      DEFAULT 0 NOT NULL,
    isRanked                        NUMBER(1, 0)      DEFAULT 0 NOT NULL,
    subjectID                       VARCHAR2(64)     NOT NULL,
    entryID                         NUMBER(38, 0)    NOT NULL,
    keyerID                         NUMBER(38, 0),
    raterID                         NUMBER(38, 0),
    classification                  NUMBER(38, 0),
    CONSTRAINT nc_assessmentData_pk PRIMARY KEY (tableID, uniqueID)
 		USING INDEX
 		(CREATE INDEX nc_assessmentData_pidx on nc_assessmentData(tableID, uniqueID)
		TABLESPACE &&tablespace_index),
    CONSTRAINT nc_assessmentData_sk  UNIQUE (tableID, uniqueID, assessmentID)
 		USING INDEX
 		(CREATE INDEX nc_assessmentData_sidx on nc_assessmentData(tableID, uniqueID, assessmentID)
		TABLESPACE &&tablespace_index)
)
TABLESPACE &&tablespace_data
;



COMMENT ON COLUMN nc_assessmentData.tableID IS 'The table identifier for all data stored within this table.  In conjunction with a unique ID creates a unique reference to all tuples within the database for tuple level access control.'
;
COMMENT ON COLUMN nc_assessmentData.uniqueID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_assessmentData.scoreOrder IS 'The sequenceStep domain is used to represent the stage (or step) in a sequence of events. For example an experimental condition (i.e. nc_expCondition) that starts on the second segment and finishes on the fourth would have a start value of 2 and an end value of 4.
'
;
COMMENT ON COLUMN nc_assessmentData.owner IS 'The database user who who owns the selected tuple.  This could be the user who created the entry or the user who manages/oversees the user who entered the tuple.'
;
COMMENT ON COLUMN nc_assessmentData.modTime IS 'The modification time of the current entry (YEAR to FRACTION(5)).'
;
COMMENT ON COLUMN nc_assessmentData.modUser IS 'The user who modified the current tuple entry.  This can be different than the owner; if the user is entering data for the owner or if a user edited a tuple.'
;
COMMENT ON COLUMN nc_assessmentData.assessmentID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_assessmentData.scoreName IS 'The name column is used to store the name of the object being described by the tuple. '
;
COMMENT ON COLUMN nc_assessmentData.scoreType IS 'The type column describes the data type of the object being described by the tuple.'
;
COMMENT ON COLUMN nc_assessmentData.textValue IS 'The textValue column is used to store the textual representations of data from all types.  This provides an easy way to return the results from queries selecting values of diffferent types without the need for conversion.'
;
COMMENT ON COLUMN nc_assessmentData.textNormValue IS 'The textValue column is used to store the textual representations of data from all types.  This provides an easy way to return the results from queries selecting values of diffferent types without the need for conversion.'
;
COMMENT ON COLUMN nc_assessmentData.encrypted IS 'The isTrue domain is used to store boolean (i.e yes/no, true/false, or 1/0) data.  The default representation of the domain is that if the value is 1, then the field is true.  The name of the field should be descriptive (e.g. is*Condition* true).'
;
COMMENT ON COLUMN nc_assessmentData.comments IS 'The description column is an open field for any comments or descriptions for a specific entry (i.e. tuple) in the database.'
;
COMMENT ON COLUMN nc_assessmentData.nc_storedAssessment_uniqueID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_assessmentData.isValidated IS 'The isTrue domain is used to store boolean (i.e yes/no, true/false, or 1/0) data.  The default representation of the domain is that if the value is 1, then the field is true.  The name of the field should be descriptive (e.g. is*Condition* true).'
;
COMMENT ON COLUMN nc_assessmentData.isRanked IS 'The isTrue domain is used to store boolean (i.e yes/no, true/false, or 1/0) data.  The default representation of the domain is that if the value is 1, then the field is true.  The name of the field should be descriptive (e.g. is*Condition* true).'
;
COMMENT ON COLUMN nc_assessmentData.subjectID IS 'The investigator ID for the research subject.'
;
COMMENT ON COLUMN nc_assessmentData.entryID IS 'The sequenceID domain is used to define an ordered sequence of entries or objects (e.g. visits by a subject).  The sequence begins with 1.'
;
COMMENT ON COLUMN nc_assessmentData.keyerID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_assessmentData.raterID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_assessmentData.classification IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
