DROP TABLE nc_extendedTuple CASCADE CONSTRAINTS
;
-- 
-- TABLE: nc_extendedTuple 
--

CREATE TABLE nc_extendedTuple(
    uniqueID         NUMBER(38, 0)    NOT NULL,
    tableID          NUMBER(38, 0)    NOT NULL,
    owner            NUMBER(38, 0)    NOT NULL,
    modTime          DATE             NOT NULL,
    modUser          NUMBER(38, 0)    NOT NULL,
    name             VARCHAR2(128)    NOT NULL,
    tupleSubClass    VARCHAR2(128)    NOT NULL,
    tupleClass       VARCHAR2(128)    NOT NULL,
    CONSTRAINT nc_extendedTuple_pk PRIMARY KEY (uniqueID)
 		USING INDEX
 		(CREATE INDEX nc_extendedTuple_pidx on nc_extendedTuple(uniqueID)
		TABLESPACE &&tablespace_index),
    CONSTRAINT nc_extendedTuple_sk  UNIQUE (uniqueID, tableID)
 		USING INDEX
 		(CREATE INDEX nc_extendedTuple_sidx on nc_extendedTuple(uniqueID, tableID)
		TABLESPACE &&tablespace_index)
)
TABLESPACE &&tablespace_data
;



COMMENT ON COLUMN nc_extendedTuple.uniqueID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_extendedTuple.tableID IS 'The table identifier for all data stored within this table.  In conjunction with a unique ID creates a unique reference to all tuples within the database for tuple level access control.'
;
COMMENT ON COLUMN nc_extendedTuple.owner IS 'The database user who who owns the selected tuple.  This could be the user who created the entry or the user who manages/oversees the user who entered the tuple.'
;
COMMENT ON COLUMN nc_extendedTuple.modTime IS 'The modification time of the current entry (YEAR to FRACTION(5)).'
;
COMMENT ON COLUMN nc_extendedTuple.modUser IS 'The user who modified the current tuple entry.  This can be different than the owner; if the user is entering data for the owner or if a user edited a tuple.'
;
COMMENT ON COLUMN nc_extendedTuple.name IS 'The name column is used to store the name of the object being described by the tuple. '
;
COMMENT ON COLUMN nc_extendedTuple.tupleSubClass IS 'The name column is used to store the name of the object being described by the tuple. '
;
COMMENT ON COLUMN nc_extendedTuple.tupleClass IS 'The name column is used to store the name of the object being described by the tuple. '
;
