DROP TABLE nc_person CASCADE CONSTRAINTS
;
-- 
-- TABLE: nc_person 
--

CREATE TABLE nc_person(
    uniqueID      NUMBER(38, 0)    NOT NULL,
    tableID       NUMBER(38, 0)    NOT NULL,
    owner         NUMBER(38, 0)    NOT NULL,
    modTime       DATE             NOT NULL,
    modUser       NUMBER(38, 0)    NOT NULL,
    first_name    VARCHAR2(128)    NOT NULL,
    last_name     VARCHAR2(128)    NOT NULL,
    email         VARCHAR2(256)    NOT NULL,
    CONSTRAINT nc_person_pk PRIMARY KEY (uniqueID)
 		USING INDEX
 		(CREATE INDEX nc_person_pidx on nc_person(uniqueID)
		TABLESPACE &&tablespace_index),
    CONSTRAINT nc_person_sk  UNIQUE (uniqueID, tableID)
 		USING INDEX
 		(CREATE INDEX nc_person_sidx on nc_person(uniqueID, tableID)
		TABLESPACE &&tablespace_index)
)
TABLESPACE &&tablespace_data
;



COMMENT ON COLUMN nc_person.uniqueID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_person.tableID IS 'The tableID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the uniqueID column these two columns specify a global unique object/tuple identifier within the database.   This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The tableID also servers as a supplementary identifier that allows each tuple''s containing table to be easily denoted.'
;
COMMENT ON COLUMN nc_person.owner IS 'The database user who who owns the selected tuple.  This could be the user who created the entry or the user who manages/oversees the user who entered the tuple.'
;
COMMENT ON COLUMN nc_person.modTime IS 'The modification time of the current entry (YEAR to FRACTION(5)).'
;
COMMENT ON COLUMN nc_person.modUser IS 'The user who modified the current tuple entry.  This can be different than the owner; if the user is entering data for the owner or if a user edited a tuple.'
;
COMMENT ON COLUMN nc_person.first_name IS 'The textValue column is used to store the textual representations of data from all types.  This provides an easy way to return the results from queries selecting values of diffferent types without the need for conversion.'
;
COMMENT ON COLUMN nc_person.last_name IS 'The textValue column is used to store the textual representations of data from all types.  This provides an easy way to return the results from queries selecting values of diffferent types without the need for conversion.'
;
COMMENT ON COLUMN nc_person.email IS 'The email column stores the email address for a person, organizational unit or institution.'
;
