DROP TABLE nc_protocol CASCADE CONSTRAINTS
;
-- 
-- TABLE: nc_protocol 
--

CREATE TABLE nc_protocol(
    protocolVersion    NUMBER(38, 0)     DEFAULT 1 NOT NULL,
    protocolID         VARCHAR2(64)     NOT NULL,
    uniqueID           NUMBER(38, 0)    NOT NULL,
    tableID            NUMBER(38, 0)    NOT NULL,
    owner              NUMBER(38, 0)    NOT NULL,
    modTime            DATE             NOT NULL,
    modUser            NUMBER(38, 0)    NOT NULL,
    name               VARCHAR2(128),
    description        CLOB,
    CONSTRAINT nc_protocol_pk PRIMARY KEY (protocolVersion, protocolID)
 		USING INDEX
 		(CREATE INDEX nc_protocol_pidx on nc_protocol(protocolVersion, protocolID)
		TABLESPACE &&tablespace_index),
    CONSTRAINT nc_protocol_sk  UNIQUE (uniqueID, tableID)
 		USING INDEX
 		(CREATE INDEX nc_protocol_sidx on nc_protocol(uniqueID, tableID)
		TABLESPACE &&tablespace_index)
)
TABLESPACE &&tablespace_data
;



COMMENT ON COLUMN nc_protocol.protocolVersion IS 'The versionID domain is used to denote version identifiers. These identifiers can have major and minor revisions.'
;
COMMENT ON COLUMN nc_protocol.protocolID IS 'The varcharID domain is used to represent alphanumeric identifiers (e.g. the ID of a research subject in a laboratory).'
;
COMMENT ON COLUMN nc_protocol.uniqueID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_protocol.tableID IS 'The tableID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the uniqueID column these two columns specify a global unique object/tuple identifier within the database.   This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The tableID also servers as a supplementary identifier that allows each tuple''s containing table to be easily denoted.'
;
COMMENT ON COLUMN nc_protocol.owner IS 'The database user who who owns the selected tuple.  This could be the user who created the entry or the user who manages/oversees the user who entered the tuple.'
;
COMMENT ON COLUMN nc_protocol.modTime IS 'The modification time of the current entry (YEAR to FRACTION(5)).'
;
COMMENT ON COLUMN nc_protocol.modUser IS 'The user who modified the current tuple entry.  This can be different than the owner; if the user is entering data for the owner or if a user edited a tuple.'
;
COMMENT ON COLUMN nc_protocol.name IS 'The name column is used to store the name of the object being described by the tuple. '
;
COMMENT ON COLUMN nc_protocol.description IS 'The description column is an open field for any comments or descriptions for a specific entry (i.e. tuple) in the database.'
;
