DROP TABLE nc_researchSubject CASCADE CONSTRAINTS
;
-- 
-- TABLE: nc_researchSubject 
--

CREATE TABLE nc_researchSubject(
    subjectID        VARCHAR2(64)     NOT NULL,
    uniqueID         NUMBER(38, 0),
    tableID          NUMBER(38, 0)    NOT NULL,
    owner            NUMBER(38, 0)    NOT NULL,
    modTime          DATE             NOT NULL,
    modUser          NUMBER(38, 0)    NOT NULL,
    extensionName    VARCHAR2(128)     DEFAULT 'researchSubject' NOT NULL,
    speciesID        NUMBER(38, 0)    NOT NULL,
    CONSTRAINT nc_researchSubject_pk PRIMARY KEY (subjectID)
 		USING INDEX
 		(CREATE INDEX nc_researchSubject_pidx on nc_researchSubject(subjectID)
		TABLESPACE &&tablespace_index),
    CONSTRAINT nc_researchSubject_sk  UNIQUE (uniqueID, tableID)
 		USING INDEX
 		(CREATE INDEX nc_researchSubject_sidx on nc_researchSubject(uniqueID, tableID)
		TABLESPACE &&tablespace_index)
)
TABLESPACE &&tablespace_data
;



COMMENT ON COLUMN nc_researchSubject.subjectID IS 'The investigator ID for the research subject.'
;
COMMENT ON COLUMN nc_researchSubject.uniqueID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_researchSubject.tableID IS 'The tableID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the uniqueID column these two columns specify a global unique object/tuple identifier within the database.   This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The tableID also servers as a supplementary identifier that allows each tuple''s containing table to be easily denoted.'
;
COMMENT ON COLUMN nc_researchSubject.owner IS 'The database user who who owns the selected tuple.  This could be the user who created the entry or the user who manages/oversees the user who entered the tuple.'
;
COMMENT ON COLUMN nc_researchSubject.modTime IS 'The modification time of the current entry (YEAR to FRACTION(5)).'
;
COMMENT ON COLUMN nc_researchSubject.modUser IS 'The user who modified the current tuple entry.  This can be different than the owner; if the user is entering data for the owner or if a user edited a tuple.'
;
COMMENT ON COLUMN nc_researchSubject.extensionName IS 'The name of the research subject type.'
;
COMMENT ON COLUMN nc_researchSubject.speciesID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
