DROP TABLE nc_transformedData CASCADE CONSTRAINTS
;
-- 
-- TABLE: nc_transformedData 
--

CREATE TABLE nc_transformedData(
    uniqueID                         NUMBER(38, 0)    NOT NULL,
    tableID                          NUMBER(38, 0)    NOT NULL,
    owner                            NUMBER(38, 0)    NOT NULL,
    modTime                          DATE             NOT NULL,
    modUser                          NUMBER(38, 0)    NOT NULL,
    nc_researchData_uniqueID         NUMBER(38, 0),
    nc_executedTransform_uniqueID    NUMBER(38, 0),
    CONSTRAINT nc_transformedData_pk PRIMARY KEY (uniqueID)
 		USING INDEX
 		(CREATE INDEX nc_transformedData_pidx on nc_transformedData(uniqueID)
		TABLESPACE &&tablespace_index),
    CONSTRAINT nc_transformedData_sk  UNIQUE (uniqueID, tableID)
 		USING INDEX
 		(CREATE INDEX nc_transformedData_sidx on nc_transformedData(uniqueID, tableID)
		TABLESPACE &&tablespace_index)
)
TABLESPACE &&tablespace_data
;



COMMENT ON COLUMN nc_transformedData.uniqueID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_transformedData.tableID IS 'The tableID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the uniqueID column these two columns specify a global unique object/tuple identifier within the database.   This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The tableID also servers as a supplementary identifier that allows each tuple''s containing table to be easily denoted.'
;
COMMENT ON COLUMN nc_transformedData.owner IS 'The database user who who owns the selected tuple.  This could be the user who created the entry or the user who manages/oversees the user who entered the tuple.'
;
COMMENT ON COLUMN nc_transformedData.modTime IS 'The modification time of the current entry (YEAR to FRACTION(5)).'
;
COMMENT ON COLUMN nc_transformedData.modUser IS 'The user who modified the current tuple entry.  This can be different than the owner; if the user is entering data for the owner or if a user edited a tuple.'
;
COMMENT ON COLUMN nc_transformedData.nc_researchData_uniqueID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_transformedData.nc_executedTransform_uniqueID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
