DROP TABLE nc_tupleData CASCADE CONSTRAINTS
;
-- 
-- TABLE: nc_tupleData 
--

CREATE TABLE nc_tupleData(
    tableID            NUMBER(38, 0)    NOT NULL,
    uniqueID           NUMBER(38, 0)    NOT NULL,
    owner              NUMBER(38, 0)    NOT NULL,
    modTime            DATE             NOT NULL,
    modUser            NUMBER(38, 0)    NOT NULL,
    extendedTupleID    NUMBER(38, 0)    NOT NULL,
    columnName         VARCHAR2(128)    NOT NULL,
    columnType         VARCHAR2(64)     NOT NULL,
    baseTableID        NUMBER(38, 0)    NOT NULL,
    baseTupleID        NUMBER(38, 0)    NOT NULL,
    textValue          VARCHAR2(255),
    comments           CLOB,
    CONSTRAINT nc_tupleData_pk PRIMARY KEY (tableID, uniqueID)
 		USING INDEX
 		(CREATE INDEX nc_tupleData_pidx on nc_tupleData(tableID, uniqueID)
		TABLESPACE &&tablespace_index),
    CONSTRAINT nc_tupleData_sk  UNIQUE (uniqueID, tableID)
 		USING INDEX
 		(CREATE INDEX nc_tupleData_sidx on nc_tupleData(uniqueID, tableID)
		TABLESPACE &&tablespace_index)
)
TABLESPACE &&tablespace_data
;



COMMENT ON COLUMN nc_tupleData.tableID IS 'The table identifier for all data stored within this table.  In conjunction with a unique ID creates a unique reference to all tuples within the database for tuple level access control.'
;
COMMENT ON COLUMN nc_tupleData.uniqueID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_tupleData.owner IS 'The database user who who owns the selected tuple.  This could be the user who created the entry or the user who manages/oversees the user who entered the tuple.'
;
COMMENT ON COLUMN nc_tupleData.modTime IS 'The modification time of the current entry (YEAR to FRACTION(5)).'
;
COMMENT ON COLUMN nc_tupleData.modUser IS 'The user who modified the current tuple entry.  This can be different than the owner; if the user is entering data for the owner or if a user edited a tuple.'
;
COMMENT ON COLUMN nc_tupleData.extendedTupleID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_tupleData.columnName IS 'The name column is used to store the name of the object being described by the tuple. '
;
COMMENT ON COLUMN nc_tupleData.columnType IS 'The type column describes the data type of the object being described by the tuple.'
;
COMMENT ON COLUMN nc_tupleData.baseTableID IS 'The tableID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the uniqueID column these two columns specify a global unique object/tuple identifier within the database.   This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The tableID also servers as a supplementary identifier that allows each tuple''s containing table to be easily denoted.'
;
COMMENT ON COLUMN nc_tupleData.baseTupleID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_tupleData.textValue IS 'The textValue column is used to store the textual representations of data from all types.  This provides an easy way to return the results from queries selecting values of diffferent types without the need for conversion.'
;
COMMENT ON COLUMN nc_tupleData.comments IS 'The description column is an open field for any comments or descriptions for a specific entry (i.e. tuple) in the database.'
;
