DROP TABLE nc_validationMethod CASCADE CONSTRAINTS
;
-- 
-- TABLE: nc_validationMethod 
--

CREATE TABLE nc_validationMethod(
    method         VARCHAR2(128)    NOT NULL,
    uniqueID       NUMBER(38, 0)    NOT NULL,
    tableID        NUMBER(38, 0)    NOT NULL,
    owner          NUMBER(38, 0)    NOT NULL,
    modTime        DATE             NOT NULL,
    modUser        NUMBER(38, 0)    NOT NULL,
    description    CLOB,
    CONSTRAINT nc_validationMethod_pk PRIMARY KEY (method)
 		USING INDEX
 		(CREATE INDEX nc_validationMethod_pidx on nc_validationMethod(method)
		TABLESPACE &&tablespace_index),
    CONSTRAINT nc_validationMethod_sk  UNIQUE (uniqueID, tableID)
 		USING INDEX
 		(CREATE INDEX nc_validationMethod_sidx on nc_validationMethod(uniqueID, tableID)
		TABLESPACE &&tablespace_index)
)
TABLESPACE &&tablespace_data
;



COMMENT ON COLUMN nc_validationMethod.method IS 'The name column is used to store the name of the object being described by the tuple. '
;
COMMENT ON COLUMN nc_validationMethod.uniqueID IS 'The uniqueID column is one of the 5 core columns (tableID, uniqueID, owner, modTime, modUser) in the database  Taken together with the tableID column these two columns specify a global unique object/tuple identifier within the database.  This unique identifying pair is critical for the extended tuple interface, database access controls and time travel. The uniqueID also servers as the principal unique identifier within each core table.'
;
COMMENT ON COLUMN nc_validationMethod.tableID IS 'The table identifier for all data stored within this table.  In conjunction with a unique ID creates a unique reference to all tuples within the database for tuple level access control.'
;
COMMENT ON COLUMN nc_validationMethod.owner IS 'The database user who who owns the selected tuple.  This could be the user who created the entry or the user who manages/oversees the user who entered the tuple.'
;
COMMENT ON COLUMN nc_validationMethod.modTime IS 'The modification time of the current entry (YEAR to FRACTION(5)).'
;
COMMENT ON COLUMN nc_validationMethod.modUser IS 'The user who modified the current tuple entry.  This can be different than the owner; if the user is entering data for the owner or if a user edited a tuple.'
;
COMMENT ON COLUMN nc_validationMethod.description IS 'The description column is an open field for any comments or descriptions for a specific entry (i.e. tuple) in the database.'
;
