drop table nc_rawdata ;

create table nc_rawdata(
    uniqueid                    bigint     not null,
    tableid                     bigint     not null,
    owner                       bigint     not null,
    modtime timestamp not null,
    moduser                     bigint     not null,
    extensionname               varchar(128)      default 'rawdata' not null,
    segmentid                   bigint     not null,
    componentid                 bigint     not null,
    nc_experiment_uniqueid      bigint     not null,
    subjectid                   varchar(64)      not null,
    datauri                     varchar(2048),
    israw                       boolean        default true not null,
    protocolversion             bigint      default 1 not null,
    protocolid                  varchar(64)      not null,
    nc_colequipment_uniqueid    bigint     not null,
    ontologysource              varchar(128)     not null,
    conceptid                   varchar(64)      not null,
    isbad                       boolean       default false,
    constraint nc_rawdata_pk primary key (uniqueid) ,
    constraint nc_rawdata_sk  unique (uniqueid, tableid) );

-- comment on column nc_rawdata.uniqueid is 'the uniqueid column is one of the 5 core columns (tableid, uniqueid, owner, modtime, moduser) in the database  taken together with the tableid column these two columns specify a global unique object/tuple identifier within the database.  this unique identifying pair is critical for the extended tuple interface, database access controls and time travel. the uniqueid also servers as the principal unique identifier within each core table.' ;

-- comment on column nc_rawdata.tableid is 'the tableid column is one of the 5 core columns (tableid, uniqueid, owner, modtime, moduser) in the database  taken together with the uniqueid column these two columns specify a global unique object/tuple identifier within the database.   this unique identifying pair is critical for the extended tuple interface, database access controls and time travel. the tableid also servers as a supplementary identifier that allows each tuple''s containing table to be easily denoted.' ;

-- comment on column nc_rawdata.owner is 'the database user who who owns the selected tuple.  this could be the user who created the entry or the user who manages/oversees the user who entered the tuple.' ;

-- comment on column nc_rawdata.modtime is 'the modification time of the current entry (year to fraction(5)).' ;

-- comment on column nc_rawdata.moduser is 'the user who modified the current tuple entry.  this can be different than the owner; if the user is entering data for the owner or if a user edited a tuple.' ;

-- comment on column nc_rawdata.extensionname is 'the extensionname column is used in inheritance hierarchies to name the object (i.e. the table) a tuple is associated with.  this should default to the name of the table.' ;

-- comment on column nc_rawdata.segmentid is 'the sequenceid domain is used to define an ordered sequence of entries or objects (e.g. visits by a subject).  the sequence begins with 1.' ;

-- comment on column nc_rawdata.componentid is 'the sequenceid domain is used to define an ordered sequence of entries or objects (e.g. visits by a subject).  the sequence begins with 1.' ;

-- comment on column nc_rawdata.nc_experiment_uniqueid is 'the uniqueid column is one of the 5 core columns (tableid, uniqueid, owner, modtime, moduser) in the database  taken together with the tableid column these two columns specify a global unique object/tuple identifier within the database.  this unique identifying pair is critical for the extended tuple interface, database access controls and time travel. the uniqueid also servers as the principal unique identifier within each core table.' ;

-- comment on column nc_rawdata.subjectid is 'the investigator id for the research subject.' ;

-- comment on column nc_rawdata.datauri is 'the uri for the selected data (can be multiple files or just a single file)' ;

-- comment on column nc_rawdata.israw is 'the istrue domain is used to store boolean (i.e yes/no, true/false, or 1/0) data.  the default representation of the domain is that if the value is 1, then the field is true.  the name of the field should be descriptive (e.g. is*condition* true).' ;

-- comment on column nc_rawdata.protocolversion is 'the versionid domain is used to denote version identifiers. these identifiers can have major and minor revisions.' ;

-- comment on column nc_rawdata.protocolid is 'the varcharid domain is used to represent alphanumeric identifiers (e.g. the id of a research subject in a laboratory).' ;

-- comment on column nc_rawdata.nc_colequipment_uniqueid is 'the uniqueid column is one of the 5 core columns (tableid, uniqueid, owner, modtime, moduser) in the database  taken together with the tableid column these two columns specify a global unique object/tuple identifier within the database.  this unique identifying pair is critical for the extended tuple interface, database access controls and time travel. the uniqueid also servers as the principal unique identifier within each core table.' ;

-- comment on column nc_rawdata.ontologysource is 'the name column is used to store the name of the object being described by the tuple. ' ;

-- comment on column nc_rawdata.conceptid is 'the varcharid domain is used to represent alphanumeric identifiers (e.g. the id of a research subject in a laboratory).' ;

-- comment on column nc_rawdata.isbad is 'the istrue domain is used to store boolean (i.e yes/no, true/false, or 1/0) data.  the default representation of the domain is that if the value is 1, then the field is true.  the name of the field should be descriptive (e.g. is*condition* true).' ;

